'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _reactAddons = require('react/addons');

var _reactAddons2 = _interopRequireDefault(_reactAddons);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _fluxEditorActions = require('../flux/EditorActions');

var _fluxEditorActions2 = _interopRequireDefault(_fluxEditorActions);

var _coreAttributes = require('../core/attributes');

var _coreDom = require('../core/dom');

var _coreReactUtils = require('../core/ReactUtils');

var _coreReactUtils2 = _interopRequireDefault(_coreReactUtils);

var T = _reactAddons2['default'].PropTypes;

exports['default'] = _reactAddons2['default'].createClass({
  displayName: 'Cursor',

  propTypes: {
    cursorPosition: T.object.isRequired,
    lineHeight: T.number.isRequired,
    cursorMotion: T.bool,
    activeAttributes: T.object,
    selectionActive: T.bool,
    focus: T.bool,
    remoteNameReveal: T.bool,
    remote: T.object
  },

  shouldComponentUpdate: function shouldComponentUpdate(nextProps) {
    // for better performance make sure objects are immutable so that we can do reference equality checks
    var propsEqual = this.props.lineHeight === nextProps.lineHeight && this.props.cursorMotion === nextProps.cursorMotion && this.props.selectionActive === nextProps.selectionActive && this.props.focus === nextProps.focus && _coreReactUtils2['default'].deepEquals(this.props.remoteNameReveal, nextProps.remoteNameReveal) && _coreReactUtils2['default'].deepEquals(this.props.cursorPosition, nextProps.cursorPosition) && _coreReactUtils2['default'].deepEquals(this.props.activeAttributes, nextProps.activeAttributes) && _coreReactUtils2['default'].deepEquals(this.props.remote, nextProps.remote, _lodash2['default'].isEqual, [function (r) {
      return r.color;
    }, function (r) {
      return r.name;
    }, function (r) {
      return r.data;
    }]);

    return !propsEqual;
  },

  getDefaultProps: function getDefaultProps() {
    return {
      remoteNameReveal: false,
      remote: null
    };
  },

  componentDidUpdate: function componentDidUpdate() {
    if (!this.caret) {
      this.caret = _reactAddons2['default'].findDOMNode(this.refs.caret);
    }
    if (this.caret) {
      var scrollByToCursor = (0, _coreDom.scrollByToVisible)(this.caret, 5, 30);
      if (scrollByToCursor.xDelta !== 0 || scrollByToCursor.yDelta !== 0) {
        window.scrollBy(scrollByToCursor.xDelta, scrollByToCursor.yDelta);
      }
    }
  },

  _remoteCursorHover: function _remoteCursorHover(e) {
    _fluxEditorActions2['default'].revealRemoteCursorName(this.props.remote);

    e.preventDefault();
    e.stopPropagation();
  },

  render: function render() {
    //console.trace('render Cursor', this.props)
    var cursorPosition = this.props.cursorPosition;
    var remote = this.props.remote;

    // the initial render before the component is mounted has no position
    if (!cursorPosition) {
      return null;
    }

    var cursorClasses = (0, _classnames2['default'])('ritzy-internal-text-cursor text-cursor', 'ritzy-internal-ui-unprintable', {
      'ritzy-internal-text-cursor-blink': !this.props.cursorMotion && !remote
    });

    var italicAtPosition = cursorPosition.position.attributes && cursorPosition.position.attributes[_coreAttributes.ATTR.ITALIC] && !remote;
    var italicActive = this.props.activeAttributes && this.props.activeAttributes[_coreAttributes.ATTR.ITALIC] && !remote;
    var italicInactive = this.props.activeAttributes && !this.props.activeAttributes[_coreAttributes.ATTR.ITALIC] && !remote;

    var caretClasses = (0, _classnames2['default'])('ritzy-internal-text-cursor-caret text-cursor-caret', {
      'ritzy-internal-text-cursor-italic': italicActive || italicAtPosition && !italicInactive
    });

    var cursorStyle = {
      left: cursorPosition.left,
      top: cursorPosition.top
    };

    if (!remote && (this.props.selectionActive || !this.props.focus)) {
      cursorStyle.opacity = 0;
      cursorStyle.visibility = 'hidden';
    } else {
      cursorStyle.opacity = 1;
    }

    var cursorHeight = Math.round(this.props.lineHeight * 10) / 10;

    if (remote) {
      var cursorTopStyle = {
        backgroundColor: remote.color,
        opacity: 1
      };
      var cursorNameStyle = {
        backgroundColor: remote.color
      };
      if (this.props.remoteNameReveal) {
        cursorTopStyle.display = 'none';
        cursorNameStyle.opacity = 1;
      } else {
        cursorNameStyle.opacity = 0;
        cursorNameStyle.display = 'none';
      }

      return _reactAddons2['default'].createElement(
        'div',
        { className: cursorClasses, style: cursorStyle },
        _reactAddons2['default'].createElement('div', { className: caretClasses, style: { borderColor: remote.color, height: cursorHeight }, onMouseOver: this._remoteCursorHover }),
        _reactAddons2['default'].createElement('div', { className: 'ritzy-internal-text-cursor-top text-cursor-top', style: cursorTopStyle, onMouseOver: this._remoteCursorHover }),
        _reactAddons2['default'].createElement(
          'div',
          { className: 'ritzy-internal-text-cursor-name text-cursor-name', style: cursorNameStyle },
          remote.name
        )
      );
    } else {
      return _reactAddons2['default'].createElement(
        'div',
        { className: cursorClasses, style: cursorStyle },
        _reactAddons2['default'].createElement('div', { className: caretClasses, style: { borderColor: 'black', height: cursorHeight }, ref: 'caret' }),
        _reactAddons2['default'].createElement('div', { className: 'ritzy-internal-text-cursor-top text-cursor-top', style: { opacity: 1 } }),
        _reactAddons2['default'].createElement('div', { className: 'ritzy-internal-text-cursor-name text-cursor-name', style: { opacity: 1 } })
      );
    }
  }

});
module.exports = exports['default'];