'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _reactAddons = require('react/addons');

var _reactAddons2 = _interopRequireDefault(_reactAddons);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _reactSpinkit = require('react-spinkit');

var _reactSpinkit2 = _interopRequireDefault(_reactSpinkit);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fluxEditorActions = require('../flux/EditorActions');

var _fluxEditorActions2 = _interopRequireDefault(_fluxEditorActions);

var _fluxEditorStore = require('../flux/EditorStore');

var _fluxEditorStore2 = _interopRequireDefault(_fluxEditorStore);

var _EditorLine = require('./EditorLine');

var _EditorLine2 = _interopRequireDefault(_EditorLine);

var _Cursor = require('./Cursor');

var _Cursor2 = _interopRequireDefault(_Cursor);

var _DebugEditor = require('./DebugEditor');

var _DebugEditor2 = _interopRequireDefault(_DebugEditor);

var _coreRichText = require('../core/RichText');

var _coreDom = require('../core/dom');

var _SwarmClientMixin = require('./SwarmClientMixin');

var _SwarmClientMixin2 = _interopRequireDefault(_SwarmClientMixin);

var _TextReplicaMixin = require('./TextReplicaMixin');

var _TextReplicaMixin2 = _interopRequireDefault(_TextReplicaMixin);

var _SharedCursorMixin = require('./SharedCursorMixin');

var _SharedCursorMixin2 = _interopRequireDefault(_SharedCursorMixin);

var _TextInput = require('./TextInput');

var _TextInput2 = _interopRequireDefault(_TextInput);

var _coreAttributes = require('../core/attributes');

var _coreEditorCommon = require('../core/EditorCommon');

var _coreReactUtils = require('../core/ReactUtils');

var _coreReactUtils2 = _interopRequireDefault(_coreReactUtils);

var _coreReplica = require('../core/replica');

var _coreTextFontMetrics = require('../core/TextFontMetrics');

var _coreTextFontMetrics2 = _interopRequireDefault(_coreTextFontMetrics);

require('../styles/internal.less');

var T = _reactAddons2['default'].PropTypes;

exports['default'] = _reactAddons2['default'].createClass({
  displayName: 'Editor',

  propTypes: {
    id: T.string.isRequired,
    fonts: T.shape({
      regular: T.object,
      bold: T.object,
      boldItalic: T.object,
      italic: T.object
    }),
    fontSize: T.number.isRequired,
    minFontSize: T.number.isRequired,
    unitsPerEm: T.number.isRequired,
    width: T.number.isRequired,
    marginH: T.number.isRequired,
    marginV: T.number.isRequired,
    userId: T.string.isRequired,
    userName: T.string,
    cursorColorSpace: T.arrayOf(T.string), // TODO allow it to be a function as well
    initialFocus: T.bool,
    wsPort: T.number
  },

  mixins: [_SwarmClientMixin2['default'], _TextReplicaMixin2['default'], _SharedCursorMixin2['default']],

  getDefaultProps: function getDefaultProps() {
    return {
      initialFocus: true,
      // The default cursor color space is a less harsh variation of the 11 Boynton colors:
      // http://alumni.media.mit.edu/~wad/color/palette.html
      // See also:
      // http://stackoverflow.com/a/4382138/430128
      // https://eleanormaclure.files.wordpress.com/2011/03/colour-coding.pdf
      cursorColorSpace: ['rgb(29, 105, 20)', // green
      'rgb(129, 38, 192)', // purple,
      'rgb(42, 75, 215)', // blue
      'rgb(41, 208, 208)', // cyan
      'rgb(173, 35, 35)', // red
      'rgb(255, 146, 51)', // orange
      'rgb(129, 197, 122)', // light green
      'rgb(157, 175, 255)', // light blue
      'rgb(255, 205, 243)', // pink
      'rgb(255, 238, 51)', // yellow
      'rgb(129, 74, 25)' // brown
      ]
    };
  },

  getInitialState: function getInitialState() {
    return _fluxEditorStore2['default'].getState();
  },

  componentWillMount: function componentWillMount() {
    _coreTextFontMetrics2['default'].setConfig(this.props);

    this._createReplica();
    _fluxEditorActions2['default'].initialize(this.props, this.replica);
  },

  componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
    _coreTextFontMetrics2['default'].setConfig(this.props);
    _fluxEditorActions2['default'].initialize(nextProps, this.replica);
  },

  componentDidMount: function componentDidMount() {
    this.clickCount = 0;
    _fluxEditorStore2['default'].listen(this.onStateChange);
  },

  shouldComponentUpdate: function shouldComponentUpdate(nextProps, nextState) {
    // for better performance make sure objects are immutable so that we can do reference equality checks
    var stateEqual = this.state.loaded === nextState.loaded && this.state.focus === nextState.focus && this.state.positionEolStart == nextState.positionEolStart && this.state.selectionActive == nextState.selectionActive && this.state.cursorMotion == nextState.cursorMotion && _coreReactUtils2['default'].deepEquals(this.state.position, nextState.position, _coreEditorCommon.charEq) && _coreReactUtils2['default'].deepEquals(this.state.selectionLeftChar, nextState.selectionLeftChar, _coreEditorCommon.charEq) && _coreReactUtils2['default'].deepEquals(this.state.selectionRightChar, nextState.selectionRightChar, _coreEditorCommon.charEq) && _coreReactUtils2['default'].deepEquals(this.state.activeAttributes, nextState.activeAttributes) && _coreReactUtils2['default'].deepEquals(this.state.remoteNameReveal, nextState.remoteNameReveal);

    if (!stateEqual) return true;

    var remoteCursorsIds = this.state.remoteCursors ? Object.keys(this.state.remoteCursors) : [];
    var nextRemoteCursorsIds = nextState.remoteCursors ? Object.keys(nextState.remoteCursors) : [];
    if (remoteCursorsIds.length !== nextRemoteCursorsIds.length) return true;
    for (var i = 0; i < remoteCursorsIds.length; i++) {
      var id = remoteCursorsIds[i];
      if (!_coreReactUtils2['default'].deepEquals(this.state.remoteCursors[id], nextState.remoteCursors[id], _lodash2['default'].isEqual, [function (r) {
        return r.color;
      }, function (r) {
        return r.name;
      }, function (r) {
        return r.data;
      }])) return true;
    }

    if (this.state.lines.length !== nextState.lines.length) return true;
    for (var i = 0; i < this.state.lines.length; i++) {
      if (!_coreReactUtils2['default'].deepEquals(this.state.lines[i], nextState.lines[i], _coreEditorCommon.linesEq)) return true;
    }

    // check props too, even though this check is fast and normally we would do faster checks first,
    // put after state checks b/c Editor props rarely change
    var propsEqual = this.props.fontSize === nextProps.fontSize && this.props.minFontSize === nextProps.minFontSize && this.props.unitsPerEm === nextProps.unitsPerEm && this.props.width === nextProps.width && this.props.marginH === nextProps.marginH && this.props.marginV === nextProps.marginV && this.props.userId === nextProps.userId && this.props.userName === nextProps.userName && this.props.initialFocus === nextProps.initialFocus && this.props.wsPort === nextProps.wsPort && _coreReactUtils2['default'].deepEquals(this.props.cursorColorSpace, nextProps.cursorColorSpace);

    return !propsEqual;
  },

  componentWillUnmount: function componentWillUnmount() {
    _fluxEditorStore2['default'].unlisten(this.onStateChange);
  },

  onStateChange: function onStateChange(state) {
    this.setState(state);
  },

  _createReplica: function _createReplica() {
    this.createTextReplica();
    this.registerCb(this._replicaInitCb, this._replicaUpdateCb);
  },

  _replicaInitCb: function _replicaInitCb(spec, op, replica) {
    // eslint-disable-line no-unused-vars
    // set our own replica for future use
    this.replicaSource = (0, _coreReplica.sourceOf)(spec);
    _fluxEditorActions2['default'].replicaInitialized();
    this.createSharedCursor();
  },

  _replicaUpdateCb: function _replicaUpdateCb(spec, op, replica) {
    // eslint-disable-line no-unused-vars
    if (this.replicaSource === (0, _coreReplica.sourceOf)(spec)) return;
    _fluxEditorActions2['default'].replicaUpdated();
  },

  _mouseEventToCoordinates: function _mouseEventToCoordinates(e) {
    // target is the particular element within the editor clicked on, current target is the entire editor div
    var targetPosition = (0, _coreDom.elementPosition)(e.currentTarget);

    return {
      x: e.pageX - targetPosition.x,
      y: e.pageY - targetPosition.y
    };
  },

  _doOnSingleClick: function _doOnSingleClick(e) {
    var coordinates = this._mouseEventToCoordinates(e);
    if (!coordinates) {
      return;
    }

    if (e.shiftKey) {
      _fluxEditorActions2['default'].selectToCoordinates(coordinates);
    } else {
      _fluxEditorActions2['default'].navigateToCoordinates(coordinates);
    }
  },

  _doOnDoubleClick: function _doOnDoubleClick() {
    _fluxEditorActions2['default'].selectWordAtCurrentPosition();
  },

  _onMouseDown: function _onMouseDown(e) {
    var _this = this;

    if (!this.state.focus) {
      _fluxEditorActions2['default'].focusInput();
    }

    if (this.clickReset) {
      clearTimeout(this.clickReset);
      this.clickReset = null;
    }
    var clickCount = this.clickCount;
    this.clickCount += 1;
    this.clickReset = setTimeout(function () {
      _this.clickCount = 0;
    }, 250);

    if (clickCount === 0) {
      this._doOnSingleClick(e);
    } else if (clickCount === 1) {
      // note that _doOnSingleClick has already executed here
      this._doOnDoubleClick(e);
    } //else if(this.clickCount === 2) // TODO handle triple-click

    e.preventDefault();
    e.stopPropagation();
  },

  _onMouseMove: function _onMouseMove(e) {
    if (e.buttons !== 1) return;

    if (!this.state.focus) {
      _fluxEditorActions2['default'].focusInput();
    }

    var coordinates = this._mouseEventToCoordinates(e);
    if (!coordinates) return;

    _fluxEditorActions2['default'].selectToCoordinates(coordinates);

    e.preventDefault();
    e.stopPropagation();
  },

  // RENDERING ---------------------------------------------------------------------------------------------------------

  _searchLinesWithSelection: function _searchLinesWithSelection(selection) {
    if (!selection) {
      selection = {
        selectionActive: this.state.selectionActive,
        selectionLeftChar: this.state.selectionLeftChar,
        selectionRightChar: this.state.selectionRightChar
      };
    }

    if (!this.state.lines || this.state.lines.length === 0 || !selection.selectionActive) {
      return null;
    }

    var left = (0, _coreEditorCommon.lineContainingChar)(this.state.lines, this.replica.getCharRelativeTo(selection.selectionLeftChar, 1, 'eof'));
    var right = (0, _coreEditorCommon.lineContainingChar)(this.state.lines.slice(left.index), selection.selectionRightChar, null);

    return {
      left: left.index,
      right: right.index + left.index
    };
  },

  _computeSelection: function _computeSelection(lineIndex, lineHeight, selection, linesWithSelection, color) {
    var _this2 = this;

    if (!selection.selectionActive) {
      return null;
    }

    // lines outside the selection range
    if (linesWithSelection && (lineIndex < linesWithSelection.left || lineIndex > linesWithSelection.right)) {
      return null;
    }

    var selectionData = function selectionData(leftX, widthX) {
      var height = Math.round(lineHeight * 10) / 10;

      // local cursor (no color) without focus is grey
      if (!color && !_this2.state.focus) {
        color = 'rgb(0, 0, 0)';
      }

      return {
        left: leftX,
        width: widthX,
        height: height,
        color: color
      };
    };

    var line = this.state.lines[lineIndex];

    // middle lines
    if (linesWithSelection && (lineIndex > linesWithSelection.left && lineIndex < linesWithSelection.right)) {
      var _selectionWidthX = line.advance;
      if (line.isEof() || line.end.char === '\n') {
        _selectionWidthX += _coreTextFontMetrics2['default'].advanceXForSpace(this.props.fontSize);
      }
      return selectionData(0, _selectionWidthX);
    }

    // last line with EOF
    if (line && line.isEof() && selection.selectionRightChar === _coreRichText.EOF) {
      return selectionData(0, _coreTextFontMetrics2['default'].advanceXForSpace(this.props.fontSize));
    }

    // empty editor (no line and selection is from BASE_CHAR to EOF)
    if (!line && (0, _coreEditorCommon.charEq)(selection.selectionLeftChar, _coreRichText.BASE_CHAR) && (0, _coreEditorCommon.charEq)(selection.selectionRightChar, _coreRichText.EOF)) {
      return selectionData(0, _coreTextFontMetrics2['default'].advanceXForSpace(this.props.fontSize));
    }

    var selectionLeftX = 0;
    var selectionWidthX = undefined;
    var selectionAddSpace = undefined;

    if (lineIndex === linesWithSelection.left) {
      // TODO change selection height and font size dynamically
      selectionLeftX = _coreTextFontMetrics2['default'].advanceXForChars(this.props.fontSize, line.charsTo(selection.selectionLeftChar));
    }

    if (lineIndex === linesWithSelection.right) {
      var selectionChars = selectionLeftX > 0 ? line.charsBetween(selection.selectionLeftChar, selection.selectionRightChar) : line.charsTo(selection.selectionRightChar);

      if (selectionChars.length === 0) {
        return null;
      }
      selectionWidthX = _coreTextFontMetrics2['default'].advanceXForChars(this.props.fontSize, selectionChars);
      selectionAddSpace = selectionChars[selectionChars.length - 1].char === '\n';
    } else {
      selectionWidthX = line.advance - selectionLeftX;
      selectionAddSpace = line.isEof() || line.end.char === '\n';
    }

    if (selectionAddSpace) {
      selectionWidthX += _coreTextFontMetrics2['default'].advanceXForSpace(this.props.fontSize);
    }

    return selectionData(selectionLeftX, selectionWidthX);
  },

  _renderLine: function _renderLine(line, index, lineHeight, localSelection, remoteSelections) {
    var _this3 = this;

    var computedSelection = this._computeSelection(index, lineHeight, localSelection.selection, localSelection.lines);
    var computedRemoteSelections = remoteSelections.map(function (s) {
      return _this3._computeSelection(index, lineHeight, s.selection, s.lines, s.color);
    }).filter(function (s) {
      return s;
    });

    return _reactAddons2['default'].createElement(_EditorLine2['default'], { key: index, line: line, lineHeight: lineHeight,
      fontSize: this.props.fontSize, selection: computedSelection, remoteSelections: computedRemoteSelections });
  },

  _cursorPosition: function _cursorPosition(lineHeight, position, positionEolStart) {
    // the initial render before the component is mounted has no position or lines
    if (!position || !this.state.lines) {
      return null;
    }

    if ((0, _coreEditorCommon.charEq)(_coreRichText.BASE_CHAR, position) || this.state.lines.length === 0) {
      return {
        position: position,
        positionEolStart: positionEolStart,
        left: this.props.marginH,
        top: this.props.marginV
      };
    }

    var result = (0, _coreEditorCommon.lineContainingChar)(this.state.lines, position, positionEolStart);
    if (!result) {
      return null;
    }
    var line = result.line;
    var index = result.index;
    var endOfLine = result.endOfLine;

    var previousLineHeights = line ? lineHeight * index : 0;

    var cursorAdvanceX = undefined;

    if (!line || endOfLine && positionEolStart && index < this.state.lines.length - 1) {
      cursorAdvanceX = 0;
    } else {
      var positionChars = line.charsTo(position);
      cursorAdvanceX = _coreTextFontMetrics2['default'].advanceXForChars(this.props.fontSize, positionChars);
    }

    return {
      position: position,
      positionEolStart: positionEolStart,
      left: this.props.marginH + cursorAdvanceX,
      top: this.props.marginV + previousLineHeights
    };
  },

  _renderInput: function _renderInput(cursorPosition) {
    var top = cursorPosition ? cursorPosition.top : 0;
    return _reactAddons2['default'].createElement(_TextInput2['default'], { id: this.props.id, ref: 'input', yPosition: top, focused: this.state.focus });
  },

  _renderCursor: function _renderCursor(cursorPosition, lineHeight, remote) {
    if (remote) {
      var id = remote._id;
      var revealName = this.state.remoteNameReveal.indexOf(id) > -1;
      return _reactAddons2['default'].createElement(_Cursor2['default'], { key: id, cursorPosition: cursorPosition, lineHeight: lineHeight,
        remoteNameReveal: revealName, remote: remote });
    } else {
      return _reactAddons2['default'].createElement(_Cursor2['default'], { key: 'local', ref: 'cursor', cursorPosition: cursorPosition, lineHeight: lineHeight,
        cursorMotion: this.state.cursorMotion, activeAttributes: this.state.activeAttributes,
        selectionActive: this.state.selectionActive, focus: this.state.focus });
    }
  },

  _renderRemoteCursors: function _renderRemoteCursors(lineHeight) {
    var _this4 = this;

    return Object.keys(this.state.remoteCursors).filter(function (id) {
      return _this4.state.remoteCursors[id].data.position;
    }).map(function (id) {
      var remoteCursor = _this4.state.remoteCursors[id];
      var remotePosition = undefined;
      try {
        remotePosition = _this4.replica.getChar(remoteCursor.data.position);
      } catch (e) {
        console.warn('Error obtaining remote position, ignoring.', e);
        return null;
      }
      // do not display remote cursor in same position as local one
      if (!_this4.state.focus || !(remoteCursor.data.positionEolStart === _this4.state.positionEolStart && (0, _coreEditorCommon.charEq)(remotePosition, _this4.state.position))) {
        var cursorPosition = _this4._cursorPosition(lineHeight, remotePosition, remoteCursor.data.positionEolStart);
        if (cursorPosition) {
          return _this4._renderCursor(cursorPosition, lineHeight, remoteCursor);
        }
      }
      return null;
    });
  },

  _renderEditorContents: function _renderEditorContents() {
    var _this5 = this;

    if (this.state.loaded) {
      var _ret = (function () {
        var lineHeight = _coreTextFontMetrics2['default'].lineHeight(_this5.props.fontSize);
        var cursorPosition = _this5._cursorPosition(lineHeight, _this5.state.position, _this5.state.positionEolStart);
        var createSelectionData = function createSelectionData(source, color) {
          var selection = {
            selectionActive: source.selectionActive,
            selectionLeftChar: source.selectionLeftChar,
            selectionRightChar: source.selectionRightChar
          };
          var linesWithSelection = _this5._searchLinesWithSelection(selection);
          return {
            selection: selection,
            lines: linesWithSelection,
            color: color
          };
        };

        var localSelection = createSelectionData(_this5.state);
        var remoteSelections = Object.keys(_this5.state.remoteCursors).filter(function (id) {
          return _this5.state.remoteCursors[id].data.selectionActive;
        }).map(function (id) {
          var remoteCursor = _this5.state.remoteCursors[id];
          return createSelectionData(remoteCursor.data, remoteCursor.color);
        });

        return {
          v: _reactAddons2['default'].createElement(
            'div',
            null,
            _this5._renderInput(cursorPosition),
            _reactAddons2['default'].createElement(
              'div',
              { className: 'ritzy-internal-text-contents text-contents', style: { position: 'relative' } },
              _this5.state.lines.length > 0 ? _this5.state.lines.map(function (line, index) {
                return _this5._renderLine(line, index, lineHeight, localSelection, remoteSelections);
              }) : _reactAddons2['default'].createElement(_EditorLine2['default'], { lineHeight: lineHeight, fontSize: _this5.props.fontSize })
            ),
            _this5._renderCursor(cursorPosition, lineHeight),
            _this5._renderRemoteCursors(lineHeight)
          )
        };
      })();

      if (typeof _ret === 'object') return _ret.v;
    } else {
      return _reactAddons2['default'].createElement(_reactSpinkit2['default'], { spinnerName: 'three-bounce', noFadeIn: true });
    }
  },

  render: function render() {
    //console.trace('render')
    var wrapperStyle = {
      width: this.props.width,
      padding: this.props.marginV + 'px ' + this.props.marginH + 'px'
    };

    return _reactAddons2['default'].createElement(
      'div',
      null,
      _reactAddons2['default'].createElement(
        'div',
        { className: 'ritzy-internal-text-content-wrapper text-content-wrapper',
          style: wrapperStyle, onMouseDown: this._onMouseDown, onMouseMove: this._onMouseMove },
        this._renderEditorContents()
      )
    );
  }

});
module.exports = exports['default'];
/*<DebugEditor editorState={this.state} replica={this.replica} searchLinesWithSelection={this._searchLinesWithSelection}/>*/