'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _reactAddons = require('react/addons');

var _reactAddons2 = _interopRequireDefault(_reactAddons);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _coreAttributes = require('../core/attributes');

var _coreEditorCommon = require('../core/EditorCommon');

var _coreReactUtils = require('../core/ReactUtils');

var _coreReactUtils2 = _interopRequireDefault(_coreReactUtils);

var _coreTextFontMetrics = require('../core/TextFontMetrics');

var _coreTextFontMetrics2 = _interopRequireDefault(_coreTextFontMetrics);

var T = _reactAddons2['default'].PropTypes;
var nbsp = String.fromCharCode(160);

exports['default'] = _reactAddons2['default'].createClass({
  displayName: 'EditorLineContent',

  propTypes: {
    line: T.object,
    fontSize: T.number.isRequired
  },

  shouldComponentUpdate: function shouldComponentUpdate(nextProps) {
    // for better performance make sure objects are immutable so that we can do reference equality checks
    var propsEqual = this.props.fontSize === nextProps.fontSize && _coreReactUtils2['default'].deepEquals(this.props.line, nextProps.line, _coreEditorCommon.linesEq);

    return !propsEqual;
  },

  _renderStyledText: function _renderStyledText(id, text, attributes) {
    var hasAttribute = (0, _coreAttributes.hasAttributeFor)(attributes);

    // vertical alignment
    var superscript = hasAttribute(_coreAttributes.ATTR.SUPERSCRIPT);
    var subscript = hasAttribute(_coreAttributes.ATTR.SUBSCRIPT);
    var verticalAlign = (0, _classnames2['default'])({
      'super': superscript,
      sub: subscript,
      baseline: !(superscript || subscript)
    });

    // font size, weight, style
    var fontSize = _coreTextFontMetrics2['default'].fontSizeFromAttributes(this.props.fontSize, attributes);
    var fontWeight = hasAttribute(_coreAttributes.ATTR.BOLD) ? 'bold' : 'normal';
    var fontStyle = hasAttribute(_coreAttributes.ATTR.ITALIC) ? 'italic' : 'normal';

    // text-decoration
    var underline = hasAttribute(_coreAttributes.ATTR.UNDERLINE);
    var strikethrough = hasAttribute(_coreAttributes.ATTR.STRIKETHROUGH);
    var textDecoration = (0, _classnames2['default'])({
      none: !(underline || strikethrough),
      underline: underline,
      'line-through': strikethrough
    });

    var style = {
      color: '#000000',
      backgroundColor: 'transparent',
      fontFamily: 'Open Sans', // TODO test other fonts, make the font selectable
      fontSize: fontSize,
      fontWeight: fontWeight,
      fontStyle: fontStyle,
      fontVariant: 'normal',
      textDecoration: textDecoration,
      verticalAlign: verticalAlign
    };

    return _reactAddons2['default'].createElement(
      'span',
      { style: style, key: id },
      text
    );
  },

  _splitIntoChunks: function _splitIntoChunks(line) {
    var _this = this;

    if (!line) return [];

    return line.chunks.map(function (chunk) {
      var chars = line.chars.slice(chunk.start, chunk.end);
      return _this._renderStyledText(chars[0].id, chars.map(function (c) {
        return c.char === ' ' ? nbsp : c.char;
      }).join(''), chunk.attributes);
    });
  },

  render: function render() {
    //console.trace('render EditorLine')
    var line = this.props.line;
    var blockHeight = 10000;
    var blockTop = _coreTextFontMetrics2['default'].top(this.props.fontSize) - blockHeight;

    return _reactAddons2['default'].createElement(
      'div',
      { className: 'ritzy-internal-text-lineview-content text-lineview-content', style: { marginLeft: 0, paddingTop: 0 } },
      _reactAddons2['default'].createElement('span', { style: { display: 'inline-block', height: blockHeight } }),
      _reactAddons2['default'].createElement(
        'span',
        { style: { display: 'inline-block', position: 'relative', top: blockTop } },
        _reactAddons2['default'].createElement(
          'span',
          { key: 'text', className: 'ritzy-internal-editor-inline-block ritzy-internal-text-lineview-text-block' },
          this._splitIntoChunks(line)
        )
      )
    );
  }

});
module.exports = exports['default'];